local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local Reload = BaikCore.Reload
local Ui = BaikCore.Ui

local FrameStyle = BaikUnitFrame.FrameStyle
local OrderNumber = BaikUnitFrame.OrderNumber

-- Add Module
local ClassColorBar = BaikUnitFrame:NewModule("ClassColorBar", "AceEvent-3.0", "AceHook-3.0")
local PlayerFrameNameBackground = nil

-- Constants
local CLASS_COLOR_DB = "health_class_color"
local COLOR_DEFAULT = true

local NAME_BACKGROUND = {
    ["target"] = TargetFrameNameBackground,
    ["focus"] = FocusFrameNameBackground
}

-- Private Methods
local function SetStatusBar(statusbar, unit)
    if (unit == statusbar.unit
        and not statusbar.lockColor
        and not statusbar.disconnected
        and UnitIsPlayer(unit)) then
        local color = Ui:GetClassColor(unit)
        statusbar:SetStatusBarColor(color.r, color.g, color.b)
    end
end

local function SetClassTexture(texture, unit)
    if UnitIsPlayer(unit) then
        local color = Ui:GetClassColor(unit)

        texture:SetVertexColor(color.r, color.g, color.b)
    end
end

-- Hook Methods
local function SyncBlizz(...)
    for unit, texture in pairs(NAME_BACKGROUND) do
        if UnitIsPlayer(unit) then
            SetClassTexture(texture, unit)
        end
    end
end

local function SyncBaik(statusbar, ...)
    SetStatusBar(statusbar, statusbar.unit)
end

local function CreatePlayerFrameNameBackground()
    local texture = PlayerFrame:CreateTexture("PlayerFrameNameBackground", BORDER)
    local color = Ui:GetClassColor("player")

    texture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-LevelBackground")
    texture:SetSize(119, 19)
    texture:SetPoint("TOPLEFT", 106, -22)
    texture:SetVertexColor(color.r, color.g, color.b, 1)

    return texture
end

local function HeaderOpts()
    return {
        name = "ClassColorBar",
        type = "header",
        order = OrderNumber:GetNext(),
    }
end

local function HealthOpts()
    return {
        name = "Class Color",
        type = "toggle",
        order = OrderNumber:GetNext(),
        get = function(...)
            return Database:Get(CLASS_COLOR_DB)
        end,
        set = function(info, value, ...)
            Database:Set(CLASS_COLOR_DB, value)
            Reload:Enable()
        end
    }
end

-- Ace3 Callbacks
function ClassColorBar:OnInitialize()
    Log:v("ClassColorBar_OnInitialize")
    -- Initialize Database``
    Database:InitIfEmpty(CLASS_COLOR_DB, COLOR_DEFAULT)

    -- Add Options
    BaikUnitFrame:AddOption("health_bar_class", HealthOpts())
end

function ClassColorBar:OnEnable()
    Log:v("ClassColorBar_OnEnable")
    if FrameStyle:IsBlizzard() then
        PlayerFrameNameBackground = CreatePlayerFrameNameBackground()
        self:RegisterEvent("GROUP_ROSTER_UPDATE", SyncBlizz)
        self:RegisterEvent("PLAYER_TARGET_CHANGED", SyncBlizz)
        self:RegisterEvent("PLAYER_FOCUS_CHANGED", SyncBlizz)
        self:RegisterEvent("UNIT_FACTION", SyncBlizz)
    else
        Hook:EnsureSecureHook(ClassColorBar, "UnitFrameHealthBar_Update", SyncBaik)
        Hook:EnsureSecureHook(ClassColorBar, "HealthBar_OnValueChanged", SyncBaik)
    end
end

function ClassColorBar:OnDisable()
    Log:v("ClassColorBar_OnDisable")
    self:UnhookAll()
end
