local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Log = BaikCore.Log
local Hook = BaikCore.Hook
local LayoutAttrib = BaikCore.LayoutAttrib

local FrameStyle = BaikUnitFrame.FrameStyle

-- Add Module
local TargetFrameMod = BaikUnitFrame:NewModule("TargetFrameMod", "AceHook-3.0")

-- UI Layout that doesn't get moved
local UNIT_FRAME = {
    TargetFrame,
    FocusFrame,
    Boss1TargetFrame,
    Boss2TargetFrame,
    Boss3TargetFrame,
    Boss4TargetFrame,
    Boss6TargetFrame,
}

local STATIC_TABLE = {
    -- NAME
    ["TextureFrameName"] = LayoutAttrib:Create(100, 12, "CENTER", -50, 36),
    -- Healthbar
    ["HealthBar"] = LayoutAttrib:Create(119, 19, "TOPRIGHT", -106, -22),
    ["TextureFrameHealthBarText"] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 18),
    ["TextureFrameHealthBarTextLeft"] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 18),
    ["TextureFrameHealthBarTextRight"] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 18),
    -- Manabr
    ["ManaBar"] = LayoutAttrib:Create(119, 12, "TOPRIGHT", -106, -41),
    ["TextureFrameManaBarText"] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 3),
    ["TextureFrameManaBarTextLeft"] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 3),
    ["TextureFrameManaBarTextRight"] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 3),
}

local LARGE_STATIC_TABLE = {
    ["HealthBar"] = LayoutAttrib:Create(119, 31, "TOPRIGHT", -106, -22),
    ["TextureFrameHealthBarText"] = LayoutAttrib:Create(nil, nil, "CENTER", -50, 7),
    ["TextureFrameHealthBarTextLeft"] = LayoutAttrib:Create(nil, nil, "LEFT", 8, 7),
    ["TextureFrameHealthBarTextRight"] = LayoutAttrib:Create(nil, nil, "RIGHT", -110, 7),
}

local LARGE_TEXTURE_SUFFIX = "%sTextureFrameTexture"

-- Private Methods
local function UnsetMinus(unitFrame)
    local classification = UnitClassification(unitFrame.unit)
    if classification == "minus" then
        Log:d("Minus detected")
        unitFrame.borderTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame")
        unitFrame.Background:SetSize(119,25);
        unitFrame.Background:SetPoint("BOTTOMLEFT", unitFrame, "BOTTOMLEFT", 7, 35);
        unitFrame.threatIndicator:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Flash")
        unitFrame.threatIndicator:SetTexCoord(0, 0.9453125, 0, 0.181640625);
        unitFrame.threatIndicator:SetWidth(242);
        unitFrame.threatIndicator:SetHeight(93);
        unitFrame.threatIndicator:SetPoint("TOPLEFT", unitFrame, "TOPLEFT", -24, 0)
    end
end

local function SetDynamic(unitFrame)
    UnsetMinus(unitFrame)
end

local function HideNameBackground()
    -- Set name background color
    for _, frame in pairs(UNIT_FRAME) do
        if frame and frame.nameBackground then
            frame.nameBackground:SetVertexColor(0, 0, 0, 0.5)
            Hook:EnsureNoOp(TargetFrameMod, frame.nameBackground, "SetVertexColor", true)
        end
    end
end

local function SyncStaticRegion(regionTable)
    -- Adjust position/dimensions
    for postfix, layout in pairs(regionTable) do
        for _, frame in pairs(UNIT_FRAME) do
            if frame then
                local syncFrame = _G[string.format("%s%s", frame:GetName(), postfix)]
                if syncFrame then
                    layout:SyncRegion(syncFrame, true)
                end
            end
        end
    end
end

local function LoadDynamic()
    Log:i("Syncing target dynamic frames")
    for _, frame in  pairs(UNIT_FRAME) do
        SetDynamic(frame)
    end
end

local function LoadStatic()
    Log:i("Syncing target static frames")

    if FrameStyle:IsCompact() then
        SyncStaticRegion(STATIC_TABLE)
        HideNameBackground()
    elseif FrameStyle:IsLarge() then
        SyncStaticRegion(LARGE_STATIC_TABLE)

        -- Loads large bar texture
        for _, frame in pairs(UNIT_FRAME) do
            if frame then
                local texture = _G[string.format(LARGE_TEXTURE_SUFFIX, frame:GetName())]
                if texture then
                    local curTexture = texture:GetTexture()
                    local newTexture = FrameStyle:GetTexture(curTexture)
                    texture:SetTexture(newTexture)
                    Hook:EnsureNoOp(TargetFrameMod, texture, "SetTexture", true)
                end
            end
        end

        HideNameBackground()
    end
end

-- Hooks
function TargetFrameMod:TargetFrame_CheckClassification(unitFrame, forceNormalTexture)
    SetDynamic(unitFrame)
end

-- Ace3 Callbacks
function TargetFrameMod:OnInitialize()
    Log:v("TargetFrameMod_OnInitialize")
end

function TargetFrameMod:OnEnable()
    Log:v("TargetFrameMod_OnEnable")
    if FrameStyle:IsBlizzard() then
        return
    end

    LoadStatic()
    LoadDynamic()
    Hook:EnsureSecureHook(self, "TargetFrame_CheckClassification")
end

function TargetFrameMod:OnDisable()
    Log:v("TargetFrameMod_OnDisable")
    self:UnhookAll()
end
